@@ -6,7 +6,7 @@ from DisplayConfigurator import DisplayConfigurator
 from main import ICON
 
 import gtk
-
+import os
 
 
 #
@@ -16,6 +16,7 @@ class Display(GlassWindow, Observable):
 
     # observer commands
     OBS_CLOSE = 0
+    OBS_RESTART = 1
 
     # mapping: str -> window type
     __WINDOW_FLAGS = {"below": GlassWindow.TYPE_KEEP_BELOW,
@@ -26,6 +27,9 @@ class Display(GlassWindow, Observable):
 
     def __init__(self, id):
 
+        # the path of the .display file
+        self.__path = os.getcwd()
+
         # the unique ID of this display
         self.__id = id
 
@@ -43,7 +47,10 @@ class Display(GlassWindow, Observable):
         self.__menu_timeout = 0
 
         # window position for detecting moves
-        self.__window_pos = (0, 0)
+        self.__window_pos = (-1, -1)
+
+        # window size for detecting resizing
+        self.__window_size = (0, 0)
 
         # mapping between sensors and targets; which targets watches
         # which sensor?
@@ -59,12 +66,13 @@ class Display(GlassWindow, Observable):
 
 
         GlassWindow.__init__(self, gtk.WINDOW_TOPLEVEL)
-        self.set_size_request(1, 1)
+        self.set_size_request(-1, -1)
 
         # set the icon
         self.set_icon(gtk.gdk.pixbuf_new_from_file(ICON))
 
         # set up event handlers
+        #self.connect("configure-event", self.__on_configure)
         self.connect("button-press-event", self.__on_button, 0)
         self.connect("button-release-event", self.__on_button, 1)
         self.connect("motion-notify-event", self.__on_motion, 0)
@@ -75,15 +83,28 @@ class Display(GlassWindow, Observable):
                         gtk.gdk.LEAVE_NOTIFY_MASK |
                         gtk.gdk.POINTER_MOTION_MASK)
 
+        # sawfish needs this
         self.realize()
         self.set_property("skip-taskbar-hint", 1)
         self.set_property("skip-pager-hint", 1)
         # don't show up in the taskbar or pager
-        gtk.idle_add(self.set_property, "skip-taskbar-hint", 1)
-        gtk.idle_add(self.set_property, "skip-pager-hint", 1)
+        #gtk.idle_add(self.set_property, "skip-taskbar-hint", 1)
+        #gtk.idle_add(self.set_property, "skip-pager-hint", 1)
 
 
 
+    def __del__(self):
+
+        print "DEL", self
+
+
+    #
+    # Returns the path of the .display file.
+    #
+    def get_path(self):  return self.__path
+    
+
+
     def add_children(self, childrendata):
 
         # create the root TargetGroup
@@ -92,7 +113,7 @@ class Display(GlassWindow, Observable):
         self.__group.add_children(childrendata)
         self.add(self.__group)
         self.__group.add_observer(self.__on_observe_group)
-
+        
 
 
     #
@@ -106,29 +127,57 @@ class Display(GlassWindow, Observable):
             configurators.append(s.get_configurator())
 
         dconf = DisplayConfigurator(configurators)
-
+        dconf.set_transient_for(self)
 
 
     #
-    # Removes this display.
+    # Removes this display. When the restart flag is set, the configuration
+    # entries will not be purged.
     #
-    def __remove_display(self):
+    def __remove_display(self, restart = 0):
 
-        self.update_observer(self.OBS_CLOSE, self.__id)
         for s in self.__sensors.values():
-            s.stop()
-
+            s.stop(restart)
+            
         del self.__sensors
+        del self.__mapping
+        self.__group._forget_parent() #destroy()
+        self.__group.destroy()
+        #del self.__group
         self.destroy()
+
+
+
+    #
+    # Reacts on configuring the window.
+    #
+    def __on_configure(self, src, event):
+
+        width = event.width
+        height = event.height
+
+        if ((width, height) != self.__window_size):
+            self.__window_size = (width, height)
+            self.set_config("width", width)
+            self.set_config("height", height)
+        #end if
+
         
 
 
+    #
+    # Reacts on closing the window.
+    #
     def __on_close(self, src, event):
 
         self.__remove_display()
-    
+        self.update_observer(self.OBS_CLOSE, self.__id)
+        
 
 
+    #
+    # Reacts on moving the mouse.
+    #
     def __on_move(self):
 
         if (self.__is_dragging):
@@ -325,7 +374,13 @@ class Display(GlassWindow, Observable):
             self.__open_configurator()
 
         elif (cmd == src.OBS_CMD_REMOVE):
-            self.__remove_display()
+            self.__remove_display(restart = 0)
+            self.update_observer(self.OBS_CLOSE, self.__id)
+
+        elif (cmd == src.OBS_CMD_RESTART):
+            self.__remove_display(restart = 1)
+            self.update_observer(self.OBS_RESTART, self.__id)
+            
 
 
 
@@ -345,8 +400,11 @@ class Display(GlassWindow, Observable):
                 self.move(ax, ay)
                 self.__window_pos = (ax, ay)
                 
-            if (w > 0 and h > 0):
+            if (w != 0 and h != 0):
                 self.resize(w, h)
+                # storing the size is useless, but it's by added by request;
+                # it makes life easy for desklets pagers
+                self.__call_sensor(["_default_:size"], [""], w, h)
 
             # don't show this window before its position has been set
             if (x != -1 and y != -1):
@@ -355,8 +413,6 @@ class Display(GlassWindow, Observable):
 
 
 
-
-
     #
     # Calls a function of a Sensor.
     #
